import arg from 'arg';
import { join } from 'path';
import { REGISTER_INSTANCE } from 'ts-node';
import chalk from 'chalk';

var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
const args = arg(
  {
    "--help": Boolean,
    "--env": String,
    "--file": String,
    "-h": "--help",
    "-e": "--env",
    "-f": "--file"
  },
  {
    permissive: true
  }
);
const runSeed = (seedBasePath) => __async(void 0, null, function* () {
  if (!process[REGISTER_INSTANCE]) {
    require("ts-node").register({ compilerOptions: { module: "commonjs" } });
  }
  require("tsconfig-paths/register");
  const seedPath = join(process.cwd(), seedBasePath);
  const dbPath = join(process.cwd(), "db/index");
  console.log(chalk.magenta("Seeding database"));
  let seeds;
  try {
    seeds = require(seedPath).default;
    if (seeds === void 0) {
      throw new Error(`Couldn't find default export from ${seedBasePath}`);
    }
  } catch (err) {
    console.log(chalk.red(`Couldn't import default from ${seedBasePath}`));
    throw err;
  }
  try {
    console.log("\nSeeding...");
    seeds && (yield seeds());
  } catch (err) {
    console.log(err);
    throw err;
  }
  const db2 = require(dbPath).default;
  yield db2.$disconnect();
  console.log("Done Seeding");
});
const db = () => __async(void 0, null, function* () {
  let filePath = "db/seeds";
  if (args["--file"]) {
    filePath = args["--file"];
  }
  if (args["--help"]) {
    console.log(`Run database commands
    ${chalk.bold("seed")} Generated seeded data in database via Prisma.
    `);
  }
  if (args["_"].slice(1)[0] && args["_"].slice(1)[0] === "seed") {
    try {
      return yield runSeed(filePath);
    } catch (err) {
      console.log(chalk.red("Could not seed database:"));
      console.log(err);
      process.exit(1);
    }
  } else {
    if (!args["--help"]) {
      console.log("\nThat command is no longer available..");
      console.log("For any prisma related commands, use the `blitz prisma` command instead:");
      console.log("\n  `blitz prisma COMMAND`\n");
    }
    return;
  }
});

export { db };
