'use strict';

const logging = require('./logging.cjs');
const routesManifest = require('./routes-manifest.cjs');
const chalk = require('chalk');
const index = require('./index.cjs');
require('tslog');
require('console-table-printer');
require('ora');
require('readline');
require('path');
require('os');
require('fs');
require('fs-extra');
require('watchpack');
require('./find-node-modules.cjs');
require('find-up');
require('findup-sync');
require('resolve-from');
require('./helpers.cjs');
require('cross-spawn');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

const chalk__default = /*#__PURE__*/_interopDefaultLegacy(chalk);

var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
const getColor = (type) => {
  switch (type) {
    case "rpc":
      return "magenta";
    case "api":
      return "blue";
    default:
      return "green";
  }
};
const routes = () => __async(void 0, null, function* () {
  const startTime = Date.now();
  process.env.BLITZ_APP_DIR = process.cwd();
  try {
    const config = routesManifest.loadConfig(process.cwd());
    const routes2 = yield routesManifest.collectAllRoutes(process.cwd(), config);
    logging.newLine();
    const table = new logging.log.Table({
      columns: [
        { name: "HTTP", alignment: "center" },
        { name: "Source File", alignment: "left" },
        { name: "Route Path", alignment: "left" },
        { name: "Type", alignment: "center" }
      ],
      sort: (q, r) => {
        if (q.Type === "PAGE" && r.Type !== "PAGE") {
          return -1;
        }
        if (q.Type !== "PAGE" && r.Type === "PAGE") {
          return 1;
        }
        if (q.Type > r.Type) {
          return 1;
        }
        if (q.Type < r.Type) {
          return -1;
        }
        return 0;
      }
    });
    routes2.forEach(({ filePath, route, verb, type }) => {
      table.addRow(
        {
          [table.table.columns[0].name]: verb.toUpperCase(),
          [table.table.columns[1].name]: filePath,
          [table.table.columns[2].name]: route,
          [table.table.columns[3].name]: type.toUpperCase()
        },
        { color: getColor(type) }
      );
    });
    console.log(table.render());
    logging.newLine();
    console.log(
      `\u2728 Done ` + chalk__default.hex("#8a3df0").bold("in ") + `${index.prettyMs(Date.now() - startTime)}`
    );
    process.exit(0);
  } catch (err) {
    console.error(err);
    process.exit(1);
  }
});

exports.routes = routes;
