import { n as newLine, l as log } from './logging.mjs';
import { l as loadConfig, c as collectAllRoutes } from './routes-manifest.mjs';
import chalk from 'chalk';
import { p as prettyMs } from './index.mjs';
import 'tslog';
import 'console-table-printer';
import 'ora';
import 'readline';
import 'path';
import 'os';
import 'fs';
import 'fs-extra';
import 'watchpack';
import './find-node-modules.mjs';
import 'find-up';
import 'findup-sync';
import 'resolve-from';
import './helpers.mjs';
import 'cross-spawn';

var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
const getColor = (type) => {
  switch (type) {
    case "rpc":
      return "magenta";
    case "api":
      return "blue";
    default:
      return "green";
  }
};
const routes = () => __async(void 0, null, function* () {
  const startTime = Date.now();
  process.env.BLITZ_APP_DIR = process.cwd();
  try {
    const config = loadConfig(process.cwd());
    const routes2 = yield collectAllRoutes(process.cwd(), config);
    newLine();
    const table = new log.Table({
      columns: [
        { name: "HTTP", alignment: "center" },
        { name: "Source File", alignment: "left" },
        { name: "Route Path", alignment: "left" },
        { name: "Type", alignment: "center" }
      ],
      sort: (q, r) => {
        if (q.Type === "PAGE" && r.Type !== "PAGE") {
          return -1;
        }
        if (q.Type !== "PAGE" && r.Type === "PAGE") {
          return 1;
        }
        if (q.Type > r.Type) {
          return 1;
        }
        if (q.Type < r.Type) {
          return -1;
        }
        return 0;
      }
    });
    routes2.forEach(({ filePath, route, verb, type }) => {
      table.addRow(
        {
          [table.table.columns[0].name]: verb.toUpperCase(),
          [table.table.columns[1].name]: filePath,
          [table.table.columns[2].name]: route,
          [table.table.columns[3].name]: type.toUpperCase()
        },
        { color: getColor(type) }
      );
    });
    console.log(table.render());
    newLine();
    console.log(
      `\u2728 Done ` + chalk.hex("#8a3df0").bold("in ") + `${prettyMs(Date.now() - startTime)}`
    );
    process.exit(0);
  } catch (err) {
    console.error(err);
    process.exit(1);
  }
});

export { routes };
